/* -------------------------------------------------------------------- */
/* SMS Client, send messages to mobile phones and pagers		*/
/*									*/
/* expect.c								*/
/*									*/
/*  Copyright (C) 1997,1998 Angelo Masci				*/
/*									*/
/*  This library is free software; you can redistribute it and/or	*/
/*  modify it under the terms of the GNU Library General Public		*/
/*  License as published by the Free Software Foundation; either	*/
/*  version 2 of the License, or (at your option) any later version.	*/
/*									*/
/*  This library is distributed in the hope that it will be useful,	*/
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/
/*  Library General Public License for more details.			*/
/*									*/
/*  You should have received a copy of the GNU Library General Public	*/
/*  License along with this library; if not, write to the Free		*/
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/
/*									*/
/*  You can contact the author at this e-mail address:			*/
/*									*/
/*  angelo@styx.demon.co.uk						*/
/*									*/
/* -------------------------------------------------------------------- */
/* $Id: expect.c,v 4.2 1998/01/17 22:49:15 root Exp $
   -------------------------------------------------------------------- */

#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>

#include "logfile.h"
#include "sms_error.h"

/* -------------------------------------------------------------------- */

#define MAX_LINE 	4096

static int caught_alarm = 0;

/* -------------------------------------------------------------------- */

static void signal_alarm(int);
static int expread(int, char *);
static int expwrite(int, char *);

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void signal_alarm(int signo)
{
	caught_alarm = 1;
	return;	
}


/* -------------------------------------------------------------------- */
/* Read explen characters from fd and copy to str,			*/
/* appending '\0'. Compare explen characters of str with expect_string.	*/
/*									*/
/* Return Values:							*/
/*									*/
/*	 0 Expect_string is found before timeout expires		*/
/*	-1 Error or Timeout exceeded					*/
/*									*/
/* -------------------------------------------------------------------- */
int expnstr(int fd, char *str, char *expect_string, int explen, int timeout)
{
	int 	i;
	char 	*ptr;

	/* ---------------------------- */
	
	caught_alarm = 0;

	if (signal(SIGALRM, signal_alarm) == SIG_ERR)
	{	exit(ESIGERR);
	}

	alarm(timeout);

	ptr = str;
	for (i=0; i<explen; i++)
	{
		if (expread(fd, ptr) != 1)
		{
			/* Timeout */

			*ptr = '\0';

			lprintf(LOG_VERBOSE, "Received: +%s+\n", str);

			alarm(0);
			return -1;
		}

		ptr++;
	}
	
	if (strncmp(str, expect_string, explen) == 0)
	{
		*ptr = '\0';

		lprintf(LOG_VERBOSE, "Received: +%s+\n", str);
		
		alarm(0);
		return 0;
	}


	*ptr = '\0';

	lprintf(LOG_VERBOSE, "Received: +%s+\n", str);

	alarm(0);
	return -1;
}



/* -------------------------------------------------------------------- */
/* Read from open file fd and search for expect_string.			*/
/* Return Values:							*/
/*									*/
/*	 0 Expect_string is found before timeout expires		*/
/*	-1 Error or Timeout exceeded					*/
/*									*/
/* -------------------------------------------------------------------- */
int expstr(int fd, char *str, char *expect_string, int maxlen, int timeout)
{
	int 	explen,
	 	i;

	char 	*ptr,
	 	*cmpptr;
	

	/* ---------------------------- */
	
	explen = strlen(expect_string);	
	caught_alarm = 0;

	if (signal(SIGALRM, signal_alarm) == SIG_ERR)
	{	exit(ESIGERR);
	}

	alarm(timeout);
	
	ptr = str;
	for (i=0; i<explen; i++)
	{
		if (expread(fd, ptr) != 1)
		{
			/* Timeout */

			*ptr = '\0';

			lprintf(LOG_VERBOSE, "Received: +%s+\n", str);

			alarm(0);
			return -1;
		}

		ptr++;
	}
	
	
	cmpptr = str;
	for( ; ; )
	{	
		if (strncmp(cmpptr, expect_string, explen) == 0)
		{
			*ptr = '\0';

			lprintf(LOG_VERBOSE, "Received: +%s+\n", str);

			alarm(0);
			return 0;
		}

		cmpptr++;
		
		if (expread(fd, ptr) != 1)
		{
			/* Timeout */

			*ptr = '\0';

			lprintf(LOG_VERBOSE, "Received: +%s+\n", str);

			alarm(0);
			return -1;
		}

		ptr++;		
	}
	

	*ptr = '\0';

	lprintf(LOG_VERBOSE, "Received: +%s+\n", str);

	alarm(0);
	return -1;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int expread(int fd, char *buf)
{
	if (caught_alarm)
	{	return -1;	/* Timeout */
	}	

	switch (read(fd, buf, 1))
	{
		case  1:
			return 1; 		/* Sucessful Read */
		case -1:
			if ((errno == EINTR) &&
			    (caught_alarm))
			{
				return -1; 	/* Timeout */
			}
			else
			{	return -2; 	/* ERROR */
			}
	}

	return 0; 	/* EOF */
}


/* -------------------------------------------------------------------- */
/* Write len bytes of buf to fd, return prematurely on timeout		*/
/* Return Values:							*/
/*									*/
/*	-1 On timeout							*/
/*	 0 Success							*/
/* -------------------------------------------------------------------- */
int twrite(int fd, char *buf, int len, int timeout)
{
	char *ptr;
	int i;
	char line[MAX_LINE];	

	/* ---------------------------- */

	caught_alarm = 0;

	if (signal(SIGALRM, signal_alarm) == SIG_ERR)
	{	exit(ESIGERR);
	}

	alarm(timeout);
	
	ptr = buf;
	for (i=0; i<len; i++)
	{
		if (expwrite(fd, ptr) != 1)
		{
			/* Timeout */

			alarm(0);
			return -1;
		}		

		ptr++;
	}


	strncpy(line, buf, len);
	line[len] = '\0';
	lprintf(LOG_VERBOSE, "Written: +%s+\n", line);

	alarm(0);
	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int expwrite(int fd, char *buf)
{
	if (caught_alarm)
	{	return -1;	/* Timeout */
	}	

	switch (write(fd, buf, 1))
	{
		case  1:
			return 1; 		/* Sucessful Write */
		case -1:
			if ((errno == EINTR) &&
			    (caught_alarm))
			{
				return -1; 	/* Timeout */
			}
			else
			{	return -2; 	/* ERROR */
			}
	}

	return 0; 	/* EOF */
}
